#!/usr/bin/perl -w
#
###############################################################################
#                                                                             #
# This program is free software: you can redistribute it and/or modify        #
# it under the terms of the GNU General Public License as published by        #
# the Free Software Foundation, either version 3 of the License, or           #
# (at your option) any later version.                                         #
#                                                                             #
# This program is distributed in the hope that it will be useful,             #
# but WITHOUT ANY WARRANTY; without even the implied warranty of              #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               #
# GNU General Public License for more details.                                #
#                                                                             #
# You should have received a copy of the GNU General Public License           #
# along with this program.  If not, see <http://www.gnu.org/licenses/>.       #
#                                                                             #
###############################################################################
#
#
# Converts ip_banishlst file for ipblocklist compatible ipset
#
# Rob Brewer 14 April 2022
#
# added Autonomous System Number lookup -rwb - 3/7/22
#


use Net::CIDR::Lite;
use strict;
use Data::Dumper;

require '/var/ipfire/Banish/Banish-functions.pl';

my $filename = "/var/ipfire/Banish/ip_Banishlist";
my $blacklist = "/srv/web/ipfire/html/banish_list";
my $location = "/usr/bin/location";
my $cache_dir = "/var/ipfire/Banish/cache";
my $dig = "/usr/bin/dig";
my @tmp = ();
my @sorted = ();

#open(OUTFILE, ">$blacklist") or die 'Unable to open Banish list file.';
#flock OUTFILE, 2;

open(INFILE, $filename) or die 'Unable to open Banish list file.';

    while ( <INFILE> )
     {
       my $line = $_;
       chomp($line);
    
	if (&Banish::validiprange($line) eq '1')
		{
		# Convert ip-ranges to cidr format for ipsets
		my $cidr = Net::CIDR::Lite->new;
     	        $cidr->add_any($line);
 	 	my @cidr_list = $cidr->list;
		push(@tmp, @cidr_list);
		}

	elsif (&Banish::validfqdn($line) eq '1')
	       {
		# convert fqdn to ip address
		chomp(my @fqdn_list = qx/$dig +short $line/);
	        push(@tmp, @fqdn_list);
	       }

	elsif (&Banish::validasn($line) eq '1')
	       {
		my $cache_file = join('/',$cache_dir,$line);

		if (-e $cache_file)
			{
			open my $cache, '<', $cache_file;
			chomp(my @asn_list = <$cache>);
			push(@tmp, @asn_list);
			close $cache;
			}
 		else
			{	      
		# remove 'AS' from line
		$line =~ s/^AS//g;
		chomp(my @asn_list = qx/$location list-networks-by-as --family=ipv4 $line/);
		push(@tmp, @asn_list);

		open my $cache, '>', $cache_file or die 'Unable to open cachefile.';
		print $cache join("\n", @asn_list);
		close ($cache);
	      		 }
	     	}
         else
	       {
                push(@tmp, $line);

	       }


     }
close(INFILE);


# Sort IPs

	@sorted = sort {
	pack('C4' => $a =~
	/(\d+)\.(\d+)\.(\d+)\.(\d+)/)
	cmp
	pack('C4' => $b =~
	/(\d+)\.(\d+)\.(\d+)\.(\d+)/)
	} @tmp;


# Remove duplicte entries

	my %duplicate = ();

	my @unique = grep{ !$duplicate{$_ }++ } @sorted; 


# Write sorted list to Banish list file

open OUTFILE, '>', "$blacklist" or die 'Unable to open Banish list file.';
flock OUTFILE, 2;
 
#       foreach (@tmp)
#       {
#	chomp;
#        print "$_\n";
#        }

print OUTFILE join ("\n", @unique);

close(OUTFILE);

